# Compute R2 (called the coefficient of determination or the proportion of
# variance explained) for hyperbola (two-phase), quadratic and linear fits to
# data for males in 17 deg treatment


# functions to compute R2 from the weights on different days for one individual

R2_hyp <- function(day, weight, w0, l1, l21, ts, s2 = 4) {
    w_hat <- w0 - l1*day - l21*0.5*((day - ts) + sqrt((day - ts)^2 + s2))
    SSres <- sum((weight - w_hat)^2)
    SStot <- sum((weight - mean(weight))^2)
    1 - SSres/SStot
}

R2_quad <- function(day, weight, hatch_day, w0, l1, l2) {
    tH <- 0.5 * hatch_day
    w_hat <- w0 + l2 * tH^2 - l1 * day - l2 * (day - tH)^2
    SSres <- sum((weight - w_hat)^2)
    SStot <- sum((weight - mean(weight))^2)
    1 - SSres/SStot
}

R2_lin <- function(day, weight, w0, l1) {
    w_hat <- w0 - l1 * day
    SSres <- sum((weight - w_hat)^2)
    SStot <- sum((weight - mean(weight))^2)
    1 - SSres/SStot
}


# data on individual weights

dat <- read.delim("postdia_data.txt")

# males from 17 deg treatment
dat0 <- subset(dat, (sex == 2) & (temp_treatment == 17))

# estimated parameters from Stan analyses
hypname <- "stan_ind_pars17_males_hyp.txt"
quadname <- "stan_ind_pars17_males_quad.txt"
linname <- "stan_ind_pars17_males_lin.txt"
dathyp <- read.delim(hypname)
datquad <- read.delim(quadname)
datlin <- read.delim(linname)

n_id <- unique(dat0$num_id)
N <- length(n_id)


# compute the R2 values for each individual

R2h <- numeric(N)
R2q <- numeric(N)
R2l <- numeric(N)

for (i in 1:N) {
    d0 <- subset(dat0, num_id == n_id[i])
    dh <- subset(dathyp, num_id == n_id[i])
    dq <- subset(datquad, num_id == n_id[i])
    dl <- subset(datlin, num_id == n_id[i])
    ts <- dh$p1_ave[1]*d0$hatch_day[1]
    R2h[i] <- R2_hyp(d0$day, d0$weight,
                     dh$w0_ave[1], dh$l1_ave[1], dh$l21_ave[1], ts)
    R2q[i] <- R2_quad(d0$day, d0$weight, d0$hatch_day[1],
                      dq$w0_ave[1], dq$l1_ave[1], dq$l2_ave[1])
    R2l[i] <- R2_lin(d0$day, d0$weight, dl$w0_ave[1], dl$l1_ave[1])
}


# mean and sd for the R2 values

R2h_ave <- mean(R2h) # 0.993306
R2q_ave <- mean(R2q) # 0.9920887
R2l_ave <- mean(R2l) # 0.9671968

R2h_sd <- sd(R2h) # 0.00515778
R2q_sd <- sd(R2q) # 0.005896179
R2l_sd <- sd(R2l) # 0.01723396


# paired t-test comparing hyp and quad
print(t.test(R2h, R2q, paired=TRUE)) # p = 0.004

# bootstrap confidence interval for hyp-quad R2 difference
library(boot)
boot.mean <- function(x, i) mean(x[i])
bthq = boot(R2h - R2q, boot.mean, R=10000)
print(bthq$t0) # 0.001217278
print(boot.ci(bthq, conf = 0.95, type = "bca")) # (0.0004, 0.0021)

# check wich cases have higher R2 for quad than for hyp
# n_id[R2q > R2h]
