# Fitting hyperbola curves to pupal weights using rstan

dat <- read.delim("postdia_data.txt")

# This file analyzes: males from 15 deg treatment
dat0 <- subset(dat, (sex == 2) & (temp_treatment == 15))
ananame <- "15_males_hyp"

# Pupal weights (in g)
w <- dat0$weight
# The day weight was measured (with day = 0 on the day pupae were taken out of
# cold to the post-diapause temperature treatment)
day <- dat0$day


# Number of data points
I <- length(w)

# For the Stan code, we need individuals in dat0 numbered from 1 to the number
# of inds, and the develpment time (variable hatch_day) for each of these

# First construct a data frame with the original individual numbers and
# corresponding development times
dat1 <- data.frame(num_id = dat0$num_id, ind_id = dat0$ind_id,
                   t_dev = dat0$hatch_day)
dat2 <- unique(dat1)

# The number of individuals
J <- length(dat2$num_id)
# Construct the new individual numbers and put into a variable in dat0
dat0$ind_id_stan <- rep(0, I)
for (k in 1:J) {
    dat0$ind_id_stan[dat0$num_id == dat2$num_id[k]] <- k
}
ind_id <- dat0$ind_id_stan

# Finally, the developmental times needed by the Stan code
t_dev <- dat2$t_dev

# List with data for Stan
PupDev.data <- list(I = I,
                    J = J,
                    w = w,
                    day = day,
                    ind_id = ind_id,
                    t_dev = t_dev
                    )


library(rstan)
set_cppo("fast")

f1 <- stan(file = "SE_UK_PupDev_hyp.stan", data = PupDev.data,
                 iter = 10, chains = 1, chain_id = 1)

# Run different stan chains in parallel
library(parallel)

# random seed to be used by all chains (this is the recommended approach; see R
# help on sflist2stanfit)
seed0 <- sample(1:1000000, 1)

sflist <-
    mclapply(1:10, mc.cores = 10,
             function(i) stan(fit=f1, data = PupDev.data,
                              seed = seed0, iter = 10000, thin = 100,
                              chains = 1, chain_id = i, refresh = -1))

f2 <- sflist2stanfit(sflist)

options(width = 150)
print(f2, digits_summary=6)
options(width = 80)

# save MCMCsamples and summaries

# for one-dimensional (top-level) parameters, save MCMC samples
f2sl <- extract(f2, c("w0_hat", "l1_hat", "l21_hat", "p1_hat",
                "sigma_w0", "sigma_l1", "sigma_l21", "sigma_p1", "sigma_w"))
df2sl <- as.data.frame(f2sl)
write.table(df2sl, paste("stan_samples", ananame, ".txt", sep = ""),
            quote = FALSE, sep = "\t", row.names = FALSE)

# plot densities of these MCMC samples
library(coda)
df2sl.mcmc <- as.mcmc(as.matrix(df2sl))
plot(df2sl.mcmc, trace = FALSE, col = "blue")
dev.off()

# for individual-level parameters, save means, meadians, and 95% conf. intervals
f2sli <- extract(f2, c("w0", "l1", "l21", "p1"))

ipars <- function(x, n, base_name) {
    x_ave <- numeric(n)
    x_med <- numeric(n)
    x_cil <- numeric(n)
    x_ciu <- numeric(n)
    for (i in 1:n) {
        x_ave[i] <- mean(x[, i])
        x_med[i] <- median(x[, i])
        x_cil[i] <- HPDinterval(as.mcmc(x[, i]))[1, "lower"]
        x_ciu[i] <- HPDinterval(as.mcmc(x[, i]))[1, "upper"]
    }
    ave <- paste(base_name, "ave", sep = "_")
    med <- paste(base_name, "med", sep = "_")
    cil <- paste(base_name, "cil", sep = "_")
    cih <- paste(base_name, "ciu", sep = "_")
    d <- data.frame(x_ave, x_med, x_cil, x_ciu)
    names(d) <- c(ave, med, cil, cih)
    d
}

dind <- data.frame(num = 1:J, num_id = dat2$num_id, ind_id = dat2$ind_id)
dw0 <- ipars(f2sli$w0, J, "w0")
dl1 <- ipars(f2sli$l1, J, "l1")
dl21 <- ipars(f2sli$l21, J, "l21")
dp1 <- ipars(f2sli$p1, J, "p1")
dipars <- data.frame(dind, dw0, dl1, dl21, dp1)
write.table(dipars, paste("stan_ind_pars", ananame, ".txt", sep = ""),
            quote = FALSE, sep = "\t", row.names = FALSE)
